#ifndef _LABEL_READER_HPP_
#define _LABEL_READER_HPP_

#include <string>
#include <opencv2/core/core.hpp>


namespace histodet {

/**
 * Class for reading label information of images (obtained for instance with
 * jafar/qdisplay/demo_suite/demo_labeler) for learning and computing detection
 * statistics.
 */
class LabelReader
{
 private:
	bool integer;
 public:
	/**
	 * Constructor
	 * @param integer round the object position at integer pixel values
	 */
	LabelReader(bool integer = true): integer(integer) {}
	/**
	 * Load a label
	 * @param filename the label file name
	 * @param[out] rect the object position
	 * @param[out] status the status of the label (0: uninitialized, 1: manual, 2: interpolated, 3: nothing)
	 * @param[out] quality the quality of the object (0: good, 1: ok, 2: bad)
	 */
	bool loadBoundingBox(std::string filename, cv::Rect &rect, int &status, int &quality);
	bool loadSizeOri(std::string filename, double &size, double &ori, double &center_x, double &center_y);
	bool loadBlobs(std::string filename, std::vector<std::pair<double,double> > &blobs);
};


} // namespace histodet


#endif

