#ifndef _PREDEFINED_DETECTORS_HPP_
#define _PREDEFINED_DETECTORS_HPP_

#include <string>
#include <opencv2/core/core.hpp>
#include "histodet/object.hpp"


namespace histodet {

/**
 * Predefined detector for the project ACTION, in order to detect the target
 * and Mana. 
 */
class PredefinedDetectorAction
{
	private:
		struct Impl;
		Impl *impl;
		
		int hist_nbits;
		
	public:
		/**
		 * Constructor
		 */
		PredefinedDetectorAction();
			
		/**
		 * Destructor
		 */
		~PredefinedDetectorAction();

		/**
		 * Intialize the detector
		 * @param hist_file xml file describing the histogram to detect the object
		 * @param refine_file file describing the refiner of the object pose
		 * @return true if succeeded
		 */
		bool init(std::string hist_file, std::string refine_file, bool rgb_order = false);

		/**
		 * Detect the object in an image
		 * @param image the image in which detection the object
		 * @param window_size size of the detection window in pixels (approximate expected object size)
		 * @param rgb_order set to true if image channels are organized in the order RGB, false if BGR
		 * @return a list of objects detected in the image
		 */
		ObjectList detect(cv::Mat const & image, int window_size);
};



} // namespace histodet


#endif

