
#include <sys/time.h>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>
#include "histodet/predefinedDetectors.hpp"

static const bool test_rgb = true;

void detect(char *histv, char *refinerv, int window_size, int nimages, char** imgv)
{
	// init
	histodet::PredefinedDetectorAction detector;
	detector.init(histv, refinerv, test_rgb);

	std::ofstream logfile;
	logfile.open("ActionDetection.csv");
	logfile << "              date| ManaDetection.target.ts| ManaDetection.target.detected| ManaDetection.target.normalized| ManaDetection.target.center[0]| ManaDetection.target.center[1]| ManaDetection.target.size| ManaDetection.target.size| ManaDetection.target.orientation| ManaDetection.target.center_sig[0]| ManaDetection.target.center_sig[1]| ManaDetection.target.size_sig| ManaDetection.target.orientation_sig|" << std::endl;

	cv::Mat image;
	struct timeval tv;
	histodet::Object obj(1);

	// process images
	for(int i = 0; i < nimages; ++i)
	{
		// load image
		if (!(image = cv::imread(imgv[i], -1)).data)
			{ std::cout << "could not load " << imgv[i] << std::endl; continue; }

		if (test_rgb) cv::cvtColor(image, image, CV_BGR2RGB);

		// detect
		histodet::ObjectList objs = detector.detect(image, window_size);
		bool detected = (objs.size() >= 1);
		if (detected) obj = objs.front();

		// write to log
		gettimeofday(&tv,NULL);
		double t = tv.tv_sec+tv.tv_usec*1e-6;
		logfile << " " << std::setw(17) << std::setfill(' ') << std::setprecision(16) << t << "|";

		char *last  = strrchr(imgv[i], '.'); if (last) *last = 0;
		char *start = strrchr(imgv[i], '/');
		logfile << " " << start+1 << "|";

		logfile << " " << std::setw(15) << std::setfill(' ') << detected << "|";
		logfile << " " << std::setw(15) << std::setfill(' ') << 0 << "|";

		bool doubled_size = false;
		for(int i = histodet::Object::lpPosU; i <= histodet::Object::lpHeading; ++i)
		{
			if (!doubled_size && i == histodet::Object::lpHeading) { i = histodet::Object::lpSize; doubled_size = true; }
			logfile << " " << std::setw(15) << std::setfill(' ') << (detected && obj.mask[i] ? obj.loc[i] : 0.0) << "|";
		}
		for(int i = histodet::Object::lpPosU; i <= histodet::Object::lpHeading; ++i)
			logfile << " " << std::setw(15) << std::setfill(' ') << (detected && obj.mask[i] ? obj.loc_std[i] : 0.0) << "|";
		logfile << std::endl;
	}
	
	logfile.close();
}


int main(int argc, char** argv)
{
	if (argc < 5) { std::cout << "Usage: histodet_demo_action <hist_file> <refiner_file> <window_size> <...images...>" << std::endl; return 1; }
	detect(argv[1], argv[2], atoi(argv[3]), argc-4, argv+4);
	return 0;
}


