
#include "histodet/predefinedDetectors.hpp"
#include "histodet/histodet.hpp"
#include "histodet/refinerHueBlobs.hpp"


namespace histodet {


struct PredefinedDetectorAction::Impl
{
	// configuration
//	typedef DistanceL1n Distance;
	typedef DistancePOSn Distance;
	typedef DetectorHistogram<HistogramDistributive<char>, Distance> DH;
	typedef ObjectExtractorFloodFill ObjectExtractor;

	// objects
	ObjectExtractor extractor;
	DH *detector;
	BucketerAbstract *bucketer;
	RefinerHueBlobs *refiner;

	Impl(): detector(NULL), bucketer(NULL), refiner(NULL) {}
	~Impl() { delete detector; delete bucketer; delete refiner; }
};


PredefinedDetectorAction::PredefinedDetectorAction()
	{ impl = new Impl(); }

PredefinedDetectorAction::~PredefinedDetectorAction()
	{ delete impl; }


bool PredefinedDetectorAction::init(std::string hist_file, std::string refine_file, bool rgb_order)
{
	delete impl->detector;
	delete impl->bucketer;
	delete impl->refiner;

	int hist_size;
 	impl->bucketer = BucketerAbstract::createBucketer(hist_file, hist_size, hist_nbits);
	impl->bucketer->setRgbOrder(rgb_order);
	impl->detector = new Impl::DH(hist_nbits, impl->bucketer);
	if (!impl->detector->detectInit(hist_file)) return false;

	impl->refiner = new RefinerHueBlobs(rgb_order);
	impl->refiner->refineInit(refine_file);
	
	return true;
}

ObjectList PredefinedDetectorAction::detect(cv::Mat const & image, int window_size)
{
	cv::Mat & res_img = impl->detector->detectImage(image, window_size);
	bool dist_inverted;
	float threshold = impl->detector->getThreshold(dist_inverted);
	ObjectList objs = impl->extractor.extract(res_img, threshold, dist_inverted, window_size);
	double score;
	bool erase = false;
	for(ObjectList::iterator it = objs.begin(); it != objs.end(); it = (erase ? objs.erase(it) : ++it))
		erase = !impl->refiner->refine(*it, image, score);

	objs.sort();
	return objs;
}


} // namespace histodet


